@@ -320,15 +320,12 @@ void Vina::compute_vina_maps(double center_x, double center_y, double center_z,
 	// Setup the search box
 	// Check first that the receptor was added
 	if (m_sf_choice == SF_AD42) {
-		std::cerr << "ERROR: Cannot compute Vina affinity maps using the AD4 scoring function.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Cannot compute Vina affinity maps using the AD4 scoring function.");
 	} else if (!m_receptor_initialized) {
 		// m_model
-		std::cerr << "ERROR: Cannot compute Vina or Vinardo affinity maps. The (rigid) receptor was not initialized.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Cannot compute Vina or Vinardo affinity maps. The (rigid) receptor was not initialized.");
 	} else if (size_x <= 0 || size_y <= 0 || size_z <= 0) {
-		std::cerr << "ERROR: Grid box dimensions must be greater than 0 Angstrom.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Grid box dimensions must be greater than 0 Angstrom");
 	} else if (size_x * size_y * size_z > 27e3) {
 		std::cerr << "WARNING: Search space volume is greater than 27000 Angstrom^3 (See FAQ)\n";
 	}
@@ -429,8 +426,7 @@ void Vina::load_maps(std::string maps) {
 void Vina::write_maps(const std::string& map_prefix, const std::string& gpf_filename,
 					  const std::string& fld_filename, const std::string& receptor_filename) {
 	if (!m_map_initialized) {
-		std::cerr << "ERROR: Cannot write affinity maps. Affinity maps were not initialized.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Cannot write affinity maps. Affinity maps were not initialized.");
 	}
 
 	szv atom_types;
@@ -461,13 +457,11 @@ std::vector< std::vector<double> > Vina::get_poses_coordinates(int how_many, dou
 	std::vector< std::vector<double> > coordinates;
 
 	if (how_many < 0) {
-		std::cerr << "Error: number of poses asked must be greater than zero.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("number of poses asked must be greater than zero.");
 	}
 
 	if (energy_range < 0) {
-		std::cerr << "Error: energy range must be greater than zero.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("energy range must be greater than zero.");
 	}
 
 	if (!m_poses.empty()) {
@@ -505,13 +499,11 @@ std::vector< std::vector<double> > Vina::get_poses_energies(int how_many, double
 	std::vector< std::vector<double> > energies;
 
 	if (how_many < 0) {
-		std::cerr << "Error: number of poses asked must be greater than zero.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("number of poses asked must be greater than zero.");
 	}
 
 	if (energy_range < 0) {
-		std::cerr << "Error: energy range must be greater than zero.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("energy range must be greater than zero.");
 	}
 
 	if (!m_poses.empty()) {
@@ -570,13 +562,11 @@ std::string Vina::get_poses(int how_many, double energy_range) {
 	std::string remarks;
 
 	if (how_many < 0) {
-		std::cerr << "Error: number of poses written must be greater than zero.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("number of poses written must be greater than zero.");
 	}
 
 	if (energy_range < 0) {
-		std::cerr << "Error: energy range must be greater than zero.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("energy range must be greater than zero.");
 	}
 
 	if (!m_poses.empty()) {
@@ -643,11 +633,9 @@ void Vina::randomize(const int max_steps) {
 	// Randomize ligand/flex residues conformation
 	// Check the box was defined
 	if (!m_ligand_initialized) {
-		std::cerr << "ERROR: Cannot do ligand randomization. Ligand(s) was(ere) not initialized.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Cannot do ligand randomization. Ligand(s) was(ere) not initialized.");
 	} else if (!m_map_initialized) {
-		std::cerr << "ERROR: Cannot do ligand randomization. Affinity maps were not initialized.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Cannot do ligand randomization. Affinity maps were not initialized.");
 	}
 
 	conf c;
@@ -666,6 +654,7 @@ void Vina::randomize(const int max_steps) {
 	VINA_FOR(i, max_steps) {
 		c = init_conf;
 		c.randomize(m_grid.corner1(), m_grid.corner2(), generator);
+		m_model.set(c);
 		penalty = m_model.clash_penalty();
 
 		if (i == 0 || penalty < best_clash_penalty) {
@@ -683,7 +672,7 @@ void Vina::randomize(const int max_steps) {
 }
 
 void Vina::show_score(const std::vector<double> energies) {
-	std::cout << "Estimated Free Energy of Binding   : " << std::fixed << std::setprecision(3) << energies[0] << " (kcal/mol) [=(1)+(2)+(3)+(4)]\n";
+	std::cout << "Estimated Free Energy of Binding   : " << std::fixed << std::setprecision(3) << energies[0] << " (kcal/mol) [=(1)+(2)+(3)-(4)]\n";
 	std::cout << "(1) Final Intermolecular Energy    : " << std::fixed << std::setprecision(3) << energies[1] + energies[2] << " (kcal/mol)\n";
 	std::cout << "    Ligand - Receptor              : " << std::fixed << std::setprecision(3) << energies[1] << " (kcal/mol)\n";
 	std::cout << "    Ligand - Flex side chains      : " << std::fixed << std::setprecision(3) << energies[2] << " (kcal/mol)\n";
@@ -762,7 +751,7 @@ std::vector<double> Vina::score(double intramolecular_energy) {
 	if (m_sf_choice == SF_VINA  || m_sf_choice == SF_VINARDO) {
 		energies.push_back(intramolecular_energy);
 	} else {
-		energies.push_back(-intra);
+		energies.push_back(intra);
 	}
 
 	return energies;
@@ -773,14 +762,11 @@ std::vector<double> Vina::score() {
 	// Check if ff and ligand were initialized
 	// Check if the ligand is not outside the box
 	if (!m_ligand_initialized) {
-		std::cerr << "ERROR: Cannot score the pose. Ligand(s) was(ere) not initialized.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Cannot score the pose. Ligand(s) was(ere) not initialized.");
 	} else if (!m_map_initialized) {
-		std::cerr << "ERROR: Cannot score the pose. Affinity maps were not initialized.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Cannot score the pose. Affinity maps were not initialized.");
 	} else if (!m_grid.is_in_grid(m_model)) {
-		std::cerr << "ERROR: The ligand is outside the grid box. Increase the size of the grid box or center it accordingly around the ligand.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("The ligand is outside the grid box. Increase the size of the grid box or center it accordingly around the ligand.");
 	}
 
 	double intramolecular_energy = 0;
@@ -844,14 +830,11 @@ std::vector<double> Vina::optimize(int max_steps) {
 	// Check if ff, box and ligand were initialized
 	// Check if the ligand is not outside the box
 	if (!m_ligand_initialized) {
-		std::cerr << "ERROR: Cannot do the optimization. Ligand(s) was(ere) not initialized.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Cannot do the optimization. Ligand(s) was(ere) not initialized.");
 	} else if (!m_map_initialized) {
-		std::cerr << "ERROR: Cannot do the optimization. Affinity maps were not initialized.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Cannot do the optimization. Affinity maps were not initialized.");
 	} else if (!m_grid.is_in_grid(m_model)) {
-		std::cerr << "ERROR: The ligand is outside the grid box. Increase the size of the grid box or center it accordingly around the ligand.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("The ligand is outside the grid box. Increase the size of the grid box or center it accordingly around the ligand.");
 	}
 
 	double e = 0;
@@ -884,14 +867,11 @@ void Vina::global_search(const int exhaustiveness, const int n_poses, const doub
 	// Vina search (Monte-carlo and local optimization)
 	// Check if ff, box and ligand were initialized
 	if (!m_ligand_initialized) {
-		std::cerr << "ERROR: Cannot do the global search. Ligand(s) was(ere) not initialized.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Cannot do the global search. Ligand(s) was(ere) not initialized.");
 	} else if (!m_map_initialized) {
-		std::cerr << "ERROR: Cannot do the global search. Affinity maps were not initialized.\n";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Cannot do the global search. Affinity maps were not initialized.");
 	} else if (exhaustiveness < 1) {
-		std::cerr << "ERROR: Exhaustiveness must be 1 or greater";
-		exit(EXIT_FAILURE);
+		throw vina_runtime_error("Exhaustiveness must be 1 or greater");
 	}
 
 	if (exhaustiveness < m_cpu) {
@@ -941,7 +921,6 @@ void Vina::global_search(const int exhaustiveness, const int n_poses, const doub
 			if (!m_no_refine & m_receptor_initialized) {
 				change g(m_model.get_size());
 				quasi_newton quasi_newton_par;
-				const vec authentic_v(1000, 1000, 1000);
 				//std::vector<double> energies_before_opt;
 				//std::vector<double> energies_after_opt;
 				int evalcount = 0;
@@ -950,7 +929,6 @@ void Vina::global_search(const int exhaustiveness, const int n_poses, const doub
 				quasi_newton_par.max_steps = unsigned((25 + m_model.num_movable_atoms()) / 3);
 
 				VINA_FOR_IN(i, poses){
-					const fl slope_orig = m_non_cache.slope;
 					VINA_FOR(p, 5){
 						m_non_cache.slope = 100 * std::pow(10.0, 2.0*p);
 						quasi_newton_par(m_model, m_precalculated_byatom, m_non_cache, poses[i], g, authentic_v, evalcount);
@@ -958,12 +936,21 @@ void Vina::global_search(const int exhaustiveness, const int n_poses, const doub
 							break;
 					}
 					poses[i].coords = m_model.get_heavy_atom_movable_coords();
-					if (!m_non_cache.within(m_model))
-						poses[i].e = max_fl;
 					m_non_cache.slope = slope;
+					// rescoring in case a ligand or flex sidechain atom is outside box
+					// ensuring poses will be sorted with the same slope (a.k.a. out of
+					// box penalty) that will be used to calculate final energies.
+					m_model.set(poses[i].c);
+					double all_grids = m_non_cache.eval(m_model, authentic_v[1]);
+					double inter_pairs = m_model.eval_inter(m_precalculated_byatom, authentic_v); // ligand -- flex
+					double intra_pairs = m_model.evalo(m_precalculated_byatom, authentic_v); // flex_i -- flex_i and flex_i -- flex_j
+					double lig_intra = m_model.evali(m_precalculated_byatom, authentic_v); // ligand_i -- ligand_i
+					poses[i].e = all_grids + inter_pairs + intra_pairs + lig_intra;
 				}
 			}
 
+			poses.sort(); // order often changes after non_cache refinement
+			m_model.set(poses[0].c);
 			if (m_no_refine || !m_receptor_initialized)
 				intramolecular_energy = m_model.eval_intramolecular(m_precalculated_byatom, m_grid, authentic_v);
 			else
@@ -992,8 +979,10 @@ void Vina::global_search(const int exhaustiveness, const int n_poses, const doub
 			}
 		}
 
-		// Since pose.e contains the final energy, we have to sort them again
-		poses.sort();
+		// In AD4, the unbound energy is intra for each pose, so order may have changed
+		// The order does not change in Vina because unbound is intra of the 1st pose
+		if (m_sf_choice == SF_AD42)
+			poses.sort();
 
 		// Now compute RMSD from the best model
 		// Necessary to do it in two pass for AD4 scoring function
@@ -1025,8 +1014,8 @@ void Vina::global_search(const int exhaustiveness, const int n_poses, const doub
 		// Clean up by putting back the best pose in model
 		m_model.set(poses[0].c);
 	} else {
-		std::cerr << "WARNING: Could not find any conformations completely within the search space.\n";
-		std::cerr << "WARNING: Check that it is large enough for all movable atoms, including those in the flexible side chains.\n";
+		std::cerr << "WARNING: Zero poses in output container after global search. This should not be happening and is likely a bug.\n";
+		std::cerr << "WARNING: If possible, please file a bug report with your input files and random seed on GitHub.\n";
 	}
 
 	// Store results in Vina object
