@@ -21,13 +21,16 @@
 */
 
 #include <iostream>
+#include <set>
 #include <string>
 #include <vector> // ligand paths
 #include <exception>
+#include "parse_error.h"
 #include <boost/program_options.hpp>
 #include "vina.h"
 #include "utils.h"
 #include "scoring_function.h"
+#include <unordered_map>
 
 struct usage_error : public std::runtime_error {
 	usage_error(const std::string& message) : std::runtime_error(message) {}
@@ -132,6 +135,7 @@ Thank you!\n";
 		double energy_range = 3.0;
 		double grid_spacing = 0.375;
 		double buffer_size = 4;
+		double unbound_energy = NAN;
 
 		// autodock4.2 weights
 		double weight_ad4_vdw   = 0.1662;
@@ -201,6 +205,7 @@ Thank you!\n";
 		advanced.add_options()
 			("score_only",     bool_switch(&score_only),     "score only - search space can be omitted")
 			("local_only",     bool_switch(&local_only),     "do local search only")
+			("unbound_energy", value<double>(&unbound_energy)->default_value(unbound_energy), "Explicitly set the Unbound System's Energy for --score_only jobs")
 			("no_refine", bool_switch(&no_refine),  "when --receptor is provided, do not use explicit receptor atoms (instead of precalculated grids) for: (1) local optimization and scoring after docking, (2) --local_only jobs, and (3) --score_only jobs")
 			("force_even_voxels", bool_switch(&force_even_voxels),  "calculated grid maps will have an even number of voxels (intervals) in each dimension (odd number of grid points)")
 			("randomize_only", bool_switch(&randomize_only), "randomize input, attempting to avoid clashes")
@@ -435,7 +440,11 @@ Thank you!\n";
 				v.write_pose(out_name);
 			} else if (score_only) {
 				std::vector<double> energies;
-				energies = v.score();
+				if (std::isnan(unbound_energy)) {
+					energies = v.score();
+				} else {
+					energies = v.score(unbound_energy);
+				}
 				v.show_score(energies);
 			} else if (local_only) {
 				std::vector<double> energies;
@@ -447,7 +456,7 @@ Thank you!\n";
 				v.write_poses(out_name, num_modes, energy_range);
 			}
 		} else if (vm.count("batch")) {
-			if (sf_name.compare("vina") == 0) {
+			if (sf_name.compare("vina") == 0 || sf_name.compare("vinardo") == 0) {
 				if (vm.count("maps")) {
 					v.load_maps(maps);
 				} else {
@@ -459,16 +468,51 @@ Thank you!\n";
 				}
 			}
 
+			std::set<std::string> repeated_names;
+			std::set<std::string> raw_names;
+			std::string name;
 			VINA_RANGE(i, 0, batch_ligand_names.size()) {
-				v.set_ligand_from_file(batch_ligand_names[i]);
+				name = get_filename(batch_ligand_names[i]);
+				if (raw_names.count(name)) {
+					repeated_names.insert(name);
+				}
+				raw_names.insert(name);
+			}
+			std::unordered_map<std::string, int> instance_counter;
 
-				out_name = default_output(get_filename(batch_ligand_names[i]), out_dir);
+			std::size_t failed_ligand_parsing = 0;
+			VINA_RANGE(i, 0, batch_ligand_names.size()) {
+				name = get_filename(batch_ligand_names[i]);
+				if (repeated_names.count(name)) {
+					if (instance_counter.count(name)) {
+						instance_counter[name] += 1;
+					} else {
+						instance_counter[name] = 1;
+					}
+					out_name = default_output(name, out_dir, instance_counter[name]);
+				} else {
+					out_name = default_output(name, out_dir);
+				}
+
+				try {
+					v.set_ligand_from_file(batch_ligand_names[i]);
+				}
+				catch(pdbqt_parse_error& e) {
+					std::cerr << e.what();
+					std::cout << "Failed parsing " << batch_ligand_names[i] << ". Skipping it.\n";
+					failed_ligand_parsing++;
+					continue;
+				}
 
 				if (randomize_only) {
 					v.randomize();
 					v.write_pose(out_name);
 				} else if (score_only) {
-					v.score();
+					if (std::isnan(unbound_energy)) {
+						v.score();
+					} else {
+						v.score(unbound_energy);
+					}
 				} else if (local_only) {
 					v.optimize();
 					v.write_pose(out_name);
@@ -477,9 +521,20 @@ Thank you!\n";
 					v.write_poses(out_name, num_modes, energy_range);
 				}
 			}
+			if (repeated_names.size()) {
+				std::cout << "Found " << repeated_names.size() << " repeated filenames in the input batch.\n";
+				std::cout << "The corresponding output filenames are suffixed with _instance<n>_out.pdbqt\n";
+			}
+			if (failed_ligand_parsing) {
+				std::cout << "Failed to parse " << failed_ligand_parsing << " ligands.\n";
+			}
 		}
 	}
 
+	catch(pdbqt_parse_error& e) {
+		std::cerr << e.what();
+		return 1;
+	}
 	catch(file_error& e) {
 		std::cerr << "\n\nError: could not open \"" << e.name.string() << "\" for " << (e.in ? "reading" : "writing") << ".\n";
 		return 1;
